import 'package:dvote/util/parsers.dart';
import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:dvote/dvote.dart';
import 'package:dvote/api/entity.dart';
import 'package:dvote/api/file.dart';
import 'package:dvote/util/content-uri.dart';

void main() {
  hdWallet();
  vocGateway();
  web3Gateway();
  dataModels();
}

void hdWallet() {
  const MethodChannel channel = MethodChannel('dvote');

  setUp(() {
    channel.setMockMethodCallHandler((MethodCall methodCall) async {
      return '0xbf0177fac1aad0cba87e5c0f54742fae36d69fff';
    });
  });

  tearDown(() {
    channel.setMockMethodCallHandler(null);
  });

  test('mnemonicToAddress', () async {
    expect(
        await mnemonicToAddress(
            'master lift organ scale accuse under head grace deny panic apart check',
            hdPath: "m/44'/60'/0'/0/0"),
        '0xbf0177fac1aad0cba87e5c0f54742fae36d69fff');
  });
}

void vocGateway() {
  final origin = "ipfs://Qmaisz6NMhDB51cCvNWa1GMS7LU1pAxdF4Ld6Ft9kZEP2a";
  final gwUri = "ws://gateway:2082/dvote";
  final publicKey =
      "02325f284f50fa52d53579c7873a480b351cc20f7780fa556929f5017283ad2449";

  test('fetchFile', () async {
    final res = await fetchFileString(ContentURI(origin), gwUri,
        gatewayPublicKey: publicKey);
    expect(res, "Hello from IPFS Gateway Checker\n");
  });
}

void web3Gateway() {
  test('web3Gateway', () async {
    final result = await fetchEntity(
      "0x180dd5765d9f7ecef810b565a2e5bd14a3ccd536c442b3de74867df552855e85",
      "0x0dCA233CE5152d58c74E74693A3C496D01542244",
      "ws://gateway:2082/dvote",
      "http://gateway:2086/web3",
      networkId: "1234",
      entryPoints: [],
    );

    print(result.writeToJson());
  });
}

void dataModels() {
  test("dataModels", () async {
    final meta =
        """{"version":"1.0","entityId":"0x409568155b5da230b6d95ca9460713b3c4d12c59851ed84aaa7ba52844c0eb5f","languages":["default"],"name":{"default":"My official entity","fr":"Mon organisation officielle"},"description":{"default":"The description of my entity goes here","fr":"La description officielle de mon organisation est ici"},"contracts":{"resolverAddress":"0x21f7DcCd9D1ce4C3685A5c50096265A8db4103b4","votingAddress":"0x1234567890123456789012345678901234567890","networkId":"goerli"},"votingProcesses":{"active":[],"ended":[]},"newsFeed":{"default":"https://hipsterpixel.co/feed.json","fr":"https://feed2json.org/convert?url=http://www.intertwingly.net/blog/index.atom"},"media":{"avatar":"https://hipsterpixel.co/assets/favicons/apple-touch-icon.png","header":"https://images.unsplash.com/photo-1557518016-299b3b3c2e7f?ixlib=rb-1.2.1&ixid=eyJhcHBfaWQiOjEyMDd9&auto=format&fit=crop&w=500&q=80"},"actions":[{"type":"browser","name":{"default":"Sign up to The Entity","fr":"S'inscrire à l'organisation"},"url":"https://cloudflare-ipfs.com/ipfs/QmZ56Z2kpG5QjJcWfhxFD4ac3DhfX21hrQ2gCTrWxzTAse","visible":"always"}],"gatewayBootNodes":[{"fetchUri":"https://bootnode:port/gateways.json","heartbeatMessagingUri":"pss://publicKey@0x0"}],"gatewayUpdate":{"timeout":60000,"topic":"vocdoni-gateway-update","difficulty":1000},"relays":[{"publicKey":"04875204f7b0bd9dfcf9af89b4fa0c44016f4b0372646c2142f55136973bd0d660565755341e4c583d49f9d607d08d40e3558de7e6f14f34aa83b7436ac70dc958","messagingUri":"<messaging-uri>"}],"bootEntities":[],"fallbackBootNodeEntities":[],"trustedEntities":[],"censusServiceManagedEntities":[]}""";
    Entity entity = parseEntity(meta);
    // Entity entity = Entity.fromJson(meta);
    expect(entity.name["default"], "My official entity",
        reason: "The name should equal My official entity");
  });
}
